######################     BURDEN MODEL    ###########################
# Use country and cluster data and fitted relationships to estimate burden

#INPUTS: estimated from burden_1.R
# source("burden_1.R")
# RP = Probability that exposure is by a rabid animal
# PP = Probability of receiving PEP given an exposure
# DP - probability that in the absence of PEP bite victim develops rabies

# DEATHS = population * (bite incidence * RP) * (1-PP) * DP * risk (i.e. endemic 1 or 0)

# DALYS - For background see:
# http://users.ugent.be/~bdvleess/DALYcalculator/publications/EUPHA_20131116_BoD_DALY.pdf
# http://www.who.int/healthinfo/global_burden_disease/metrics_daly/en/
# http://www.who.int/healthinfo/global_burden_disease/daly_disability_weight/en/
# Note GBD2010 uses prevalence rather than incidence (equivalent for rabies)

source("burden_1.R")
require("Hmisc")
require("VGAM")
library(LifeTable)

YLLcalc <- function(LTvar, LTvalues, cause, interval=c(1, 4, rep(5,19)), 
                    discount = 0.03, C = 0.1658, Beta = 0.04, alpha = 1){    
  # Calculates YLL (age-weighted and discounted) from rabies or Adverse Events (AE) from Nerve Tissue Vaccines (NTVs)
  # Inputs: mortality rate (country-specific) and cause of death (rabies or NTV) and interval for life tables
  #         If Time discounting: discount = 0.03
  #         If Age-weighting (alpha=1): C = 0.1658 & Beta = 0.04. If no age-weighting (alpha=0): C = 1 & Beta = 0
  # Returns: Annual YLL for avg death in country of interest (to be multiplied by the number of deaths)
  
  mortality_year <- rep(LTvalues, interval)
  if(LTvar == "Mx") { LE = LT(Mx = mortality_year, axmethod = "keyfitz", mxsmooth = FALSE)$Tx }
  if(LTvar == "LE") { LE = mortality_year }
  
  curr_age <- seq(from = 0, by = 1, length.out = length(mortality_year))
  death_age <- round(curr_age + LE)
  
  cost <- sapply(1:(length(curr_age)), function(i){
    ages <- curr_age[i]:death_age[i]
    future <- (1:length(ages))-1
    disc_year <- exp(-discount*(future))
    weighting <- C*ages^alpha*exp(-Beta*ages)
    return(sum(disc_year*weighting))
  })
  
  if(cause == "rabies"){death_pc <- DALYrabies$death_pc/sum(DALYrabies$death_pc)}
  if(cause == "vaccine"){death_pc <- DALYrabies$exposure_pc/sum(DALYrabies$exposure_pc)}
  
  burden_year <- rep(death_pc/interval, interval)
  return(sum(cost*burden_year))
}

# TESTING 
# YLLcalc("Mx", t(pop[i,6:26]), "rabies") 
# YLLcalc(LTvar="LE", LTvalues=GBD2010$LE, cause="rabies", discount = 0, C = 1, Beta = 0, alpha = 0) 

# Function to calculate YLD due to AEs from NTVs and anxiety - 
# no TD or AW, because disability durations < 1 year therefore negligible

YLDcalc <- function(duration, disability_weight, probability){
  # Calculates YLD from duration, weight and probability of disability
  # Returns: YLD for avg bite victim if given NTV in country of interest (to be multiplied by the number of PEP)
  return(duration * disability_weight * probability)
}


# INPUT VALUES AND TABLES (generated by burden_1.R):
DP <- 0.19 # from Shim et al. 2009
DPl <- 0.13; DPu <- 0.28

rrp = binconf(55, 74) # Rabies recognition probability and CIs - from Lembo et al. 08
RRP <- rrp[1]
RRPl <- rrp[2]; RRPu <- rrp[3]

pop <- read.csv("CountriesPop.csv")  # http://esa.un.org/wpp (country-specific mortality rates, % urban etc)

# DALYs - disability weightings and lifetables
DALYrabies <- read.csv("DALY_params_rabies.csv")  # Probability of rabies death at each age class
DALYvacc <- read.csv("DALY_params_vaccination.csv")
GBD2010 <- read.csv("GBD2010_LE.csv")  # GBD 2010 study - LE of 86 years at birth (based on highest observed LE per age group)
west26 <- read.csv("West26.csv")  # Standard West 26 lifetable # Coale-Demeny Model Life Tables West (26) - Life Expectancy at Birth = 80∙0 for males & 82∙5 for females

########################################################################
# DEATHS, EXPOSURES & PEP ADMINISTRATION - basis for sensitivity analyses
d$deaths <- with(d, RISK * pop * BI * mRP * (1-PP) * DP); sum(d$deaths, na.rm=T) 
d$DR = d$deaths/d$Human_population_2010
d$exp <- with(d, RISK * pop * BI * mRP)  # Genuine Exposures - i.e. to rabid (not healthy) animals
d$PEP <- with(d, RISK * pop * BI * PP)  # PEP  - all bites, does not distinguish rabid vs healthy animals
d$prev_death <- with(d, RISK * pop * BI * mRP * PP * DP)  # Deaths prevented

# DALYs:
# YLL due to rabies and use of NTVs
for(i in 1:length(pop$COUNTRY)){   
  
  # rabies deaths  
  d$YLL[i] = YLLcalc("Mx", t(pop[i,6:26]), "rabies") * d$deaths[i]  # Country-specific (comparable to Knobel et al. 2005)   
  d$YLLwest[i] = YLLcalc("LE", west26$LE, "rabies") * d$deaths[i]  # Uses standard West 26 LE
  d$YLLle[i] = YLLcalc(LTvar="LE", LTvalues=GBD2010$LE, cause="rabies", discount = 0, C = 1, Beta = 0, alpha = 0) * d$deaths[i]  # GBD2010 use of LE (could also look at HALE - GBD2010$hale)

  # deaths from Semple vaccine use
  d$semple_d[i] = YLLcalc("Mx", t(pop[i,6:26]), "vaccine") * with(d, PEP[i]*NTV[i]*SMP[i]) * DALYvacc$Percentage[1] 
  d$semple_d_west[i] = YLLcalc("LE", west26$LE, "vaccine") * with(d, PEP[i]*NTV[i]*SMP[i]) * DALYvacc$Perc[1]  
  d$semple_d_le[i] = YLLcalc(LTvar="LE", LTvalues=GBD2010$LE, cause="vaccine", discount = 0, C = 1, Beta = 0, alpha = 0) * with(d, PEP[i]*NTV[i]*SMP[i]) * DALYvacc$Perc[1]  

  # deaths from SMB vaccine use
  d$SMB_d[i] = YLLcalc("Mx", t(pop[i,6:26]), "vaccine") * with(d, PEP[i]*NTV[i]*SMB[i]) * DALYvacc$Percentage[2]  
  d$SMB_d_west[i] = YLLcalc("LE", west26$LE, "vaccine") * with(d, PEP[i]*NTV[i]*SMB[i]) * DALYvacc$Perc[2]  
  d$SMB_d_le[i] = YLLcalc(LTvar="LE", LTvalues=GBD2010$LE, cause="vaccine", discount = 0, C = 1, Beta = 0, alpha = 0) * with(d, PEP[i]*NTV[i]*SMB[i]) * DALYvacc$Perc[2]  
}

# Compare:
sum(d$YLL, na.rm = T); sum(d$YLLwest, na.rm = T); sum(d$YLLle, na.rm = T)
sum(d$semple_d, na.rm = T); sum(d$semple_d_west, na.rm = T); sum(d$semple_d_le, na.rm = T)
sum(d$SMB_d, na.rm = T); sum(d$SMB_d_west, na.rm = T); sum(d$SMB_d_le, na.rm = T)


# YLD from AEs due to NTVs with Semple and SMB vaccines (NTVs)
d$AE_semple = YLDcalc(DALYvacc$duration[3], DALYvacc$level[3], DALYvacc$Percentage[3]) * with(d, PEP*NTV*SMP) # Semple vaccines
d$AE_SMB = YLDcalc(DALYvacc$duration[4], DALYvacc$level[4], DALYvacc$Percentage[4]) * with(d, PEP*NTV*SMB)  # SMB

#Overall DALYs from NTV use
d$NTVgbd = with(d, semple_d_le + SMB_d_le + AE_semple + AE_SMB)  # standard new LT, No TD or AW (comparable to GBD 2010)
d$NTVknobel = with(d, semple_d + SMB_d + AE_semple + AE_SMB)  # Country specific LT,  3%TD & AW (comparable to Knobel et al. 2005)

# YLD due to exposure - assume only those that sought and obtained PEP were anxious! 
d$Anxiety = YLDcalc(DALYvacc$duration[5], DALYvacc$level[5], DALYvacc$Percentage[5]) * d$PEP

# ALL DALYS - excluding Anxiety
d$DALYgbd = apply(cbind(d$YLLle, d$NTVgbd), 1, sum, na.rm=T); sum(d$DALYgbd) # DALY total (GBD2010)
d$DALYknobel = apply(cbind(d$YLL, d$NTVknobel), 1, sum, na.rm=T) # DALY total with 3%TD & AW (Knobel et al. 05)

#########################################################################
# COSTS 
# DIRECT - proportion of patients given PEP regimen, unit cost & consultation costs 
Ig = with(d, PEP * ((HRIG * HRIGcost) + (ERIG * ERIGcost)))  # RIG
NTV = with(d, PEP * NTV * ((NTVcost * ND_NTV) + (FIRST_CONSULT + ((ND_NTV-1)*SUB_CONSULT))) )  # NTV
IM = with(d, PEP * TCV_IM * ((TCVcost * ND_IM) + ( ((ND_IM-1)*SUB_CONSULT)+FIRST_CONSULT )) )  # IM
dpv = 4  # 4 ID doses in 1 vial
ID = with(d, PEP * ((TCV_ID * TCVcost/dpv * ND_ID) + (TCV_ID * ((ND_ID-1)*SUB_CONSULT)+FIRST_CONSULT))) # ID
d$direct = apply( cbind(Ig, NTV, IM, ID), 1, sum, na.rm = T); sum(d$direct, na.rm = T)

# TRAVEL	
n_trips = with(d, (NTV * ND_NTV) + (TCV_IM * ND_IM) + (TCV_ID * ND_ID)) 
trip_cost = d$TRAVEL * ((pop$Urban * d$TRAV_U) + ((1-pop$Urban) * d$TRAV_R))
d$travel = d$PEP * n_trips * trip_cost; sum(d$travel, na.rm = T)  #Note do not include accompanying guide

d$lost_income = n_trips * d$PEP * d$GDP.US./365  # ST Lost income
d$prem_death = d$YLLle * d$GDP.US.  # Premature deaths - use GBD2010 lifetables to get YLL   
d$dog_vacc = d$Human * d$DVAC * d$DVAC_C  # Dog vaccination
d$dpm = with(d, Human_population_2010 * ((DSTE*DSTE_C) + (DKIL*DKIL_C)))  # Dog pop mgmt
d$Livestock = with(d, Human_population_2010 * LLpc * L.cost.per)  # Livestock losses 
d$surveillance = d$Human * d$TEST * d$TEST_C  # Surveillance

# TOTAL COSTS	
costs = d[,c("direct", "travel", "lost_income", "prem_death", "dog_vacc", "dpm", "Livestock", "surveillance")]
cat_costs = apply(costs, 2, sum, na.rm=TRUE) # by category
country_costs = apply(costs, 1, sum, na.rm=TRUE) # by country
total_costs = sum(country_costs)

write.csv(d, "burden.csv", row.names=FALSE)

